#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import division

import numpy
from scipy import *
from scipy import optimize
import scipy.fftpack
import scipy.special
from pylab import * 
from scipy.linalg import *
import copy
import time

import sys
import particle_tools as my

#import pdb # debugger
# insert pdb.set_trace() wherever you want to debug

# constants 
eta1 = array([1.64493, 1.20206, 1.08232, 1.03693, 1.01734, 1.00835, 1.00408, 1.00201,
1.00099, 1.00049, 1.00025, 1.00012, 1.00006, 1.00003, 1.00002, 1.00001, 1.0, 1.0])
eta1 = eta1 * (2**arange(2,20)-1)
eta2 = arange(3,21)
eta = eta1/eta2
LOG2 = log(2.)
LOG2PI = log(2*pi)
INV2PI = 1/(2*pi)
LOGSQRT2PI = 0.5*log(2*pi)

def transformd(d):
    " transform d in [0,0.5]  into x in (-infty,+infty) "
    # if you change this, propagate invtransformd and logprior accordingly 
    return -log(1./(2.*d)-1.)

def invtransformd(x):
    " inverse transform "
    # if you change this, propagate invtransformd and logprior accordingly
    return 0.5/(1.+exp(-x)) # reparametrisation

def appro(d,k=100):
    " appro by Brunero: note k must be >21 "
    somma = (d**arange(21,k+1))*(2.**arange(20,k)-1)
    diviso = arange(21,k+1)
    ve = somma/diviso
    return sum(ve)

def ESS(w):
    " Effective Sample Size "
    return sum(w)**2/sum(w**2)

def ESSlog(lw):
    " ESS of exp(lw)"
    w = exp(lw-max(lw))
    return ESS(w)

def logmean(x):
    " returns log(mean(e^x)) " 
    mx = max(x)
    return mx+log(mean(exp(x-mx)))

#*************************
class Context:
    """ context contains all the settings of the algorithm,
    including prior hyperparameters, number of iterations, etc
    """
    def __init__(self,data,kmax=20):
        # Prior hyper-parameters
        self.kmax = kmax # max value of k, so that arrays are of max size kmax
        q = 0.2 # shifted geometric(pk) prior for k (ie Geo for k+1) 
        self.piq = q 
        self.pik = q*(1.-q)**arange(self.kmax+1)
        self.pik *= 1./sum(self.pik)
        self.logpik = log(self.pik)
        # prior for the Fourier coefficients thetai
        self.mutheta = 0.
        self.sigmatheta = 10./arange(1,self.kmax+1) # as in J&B's program
        # inv gamma Prior for sigma^2
        self.asigma, self.bsigma = 0.5, 0.5 # corresponds to ep/2 in Ju & Bru's program
        # N(0,sigma^2/g) prior for mu (cond on sigma)
        self.gmu = 0.1
        # Data
        self.y = data
        self.ndata = data.size
        self.sy = empty(self.ndata)
        self.sy[0] = sum(data**2)
        periodo = abs( fft(concatenate((zeros(1),self.y))) )**2 /(2.*pi*self.ndata)
        self.periodogram = periodo[1:] # 0-th element is not needed 
        # NOT NEEDED ANY MORE
        for i in range(1,self.ndata):
            self.sy[i] = 2.*sum(data[:-i]*data[i:])
        # Misc
        self.eps = 1e-6
        self.MinusInf = -1e100
        self.nmovesteps = 5 # how many MCMC steps after resampling
        self.nmovesteps_final= 50 # how many move steps at the last iteration
        self.threshold_ESS = 0.5 #when ESS<threshold*N, resample-move step is triggered
        self.adaptRJ = False # do we adapt the jump probabilities
        self.HMscale = 2.38**2 # scale factor for HMRW (square because variance)
        # note that scale for dimension d=k+1 is 2.38/sqrt(d)
        self.IS_RJBD = True # birth-death reversible jump
        self.IS_RJG = False # reversible jump using Green (2003)'s strategy
    # note: if both are True, two steps are performed!
        self.IS_POSDIS = False # positive discrimination, ie Wang-Landau wrt K
# WARNING: POSDIS is not compatible with RJ steps


#*************************
class Parameter:
    """ Defines structure of parameter
    """
    def __init__(self,k):
        self.k = k
        self.theta = zeros(k)
        self.td = 0.
        self.l = 0.
        self.llik = 0. 
            
    def d(self):
        return invtransformd(self.td)

    def disp(self):
        print "k: ",self.k," ,d: ",invtransformd(self.td),", theta: ",self.theta, "l", self.l, "ll", self.llik

    def invspectrum(self,t):
        """ inverse of Spectral density (divided by 4\pi^2)
        evaluated at t (t is a vector, and
        ouptut is a vector of same size)
        corresponding to parameter self
        """
        return INV2PI*( (2.*(1.-cos(t)))**(self.d())*
        exp(-sum(self.theta*cos(multiply.outer(t,arange(1,self.k+1))),axis=1)) )
                                        
    def modifspectrum(self,t):
        """ true spectral density, but modified so that it 
        does not goes to infinity at 0; when computing the gamma function, 
        a correcting term must be added, see truegamma
        """
        s = INV2PI*(2.*(1.-cos(t)))**(-self.d())*(
        exp(sum(self.theta*cos(multiply.outer(t,arange(1,self.k+1))),axis=1)) 
        -exp(sum(self.theta)) )
        s[t==0] = 0.
        return s 

    def gamma(self,n,M = None, inv = True):
        """
        compute the gamma(k), k=0,...,n-1 using p.590 of of Num. Recipes in C
        """
        def W(t):
                """
                Function W(theta) as defined p. 586 of Num. Recipes in C
                                (Trapeozidal order)
                """
                return -2*alpha0(t)

        def alpha0(t):
                " idem, function alpha0 (but only real part) " 
                return where(t==0,-0.5,-(1.-cos(t))/t**2)

        if M == None:
                M = 2 ** (1+ceil(log2(n))) # M is a power of 2, such that M>=2*n
        gridpos = linspace(0.,pi,1+M/2) # grid (positive part, on [0,pi])
        if inv:
            hpos = self.invspectrum(gridpos)
        else:
            hpos = self.modifspectrum(gridpos)
        h = concatenate((hpos[:0:-1],hpos)) #spectrum is an even function
        Four = scipy.fftpack.rfft(h[:-1],M,overwrite_x=1) 
        F = empty(n)
        F[0] = Four[0]
        F[1:] = Four[1:2*n-1:2]
        theta = gridpos[:n]
        gamma = (2*pi/M)*(W(theta)*F) #+2*h[0]*alpha0(theta))
        gamma[1::2] = -gamma[1::2] # multiply by exp(-im\pi)=(-1)^m
        return gamma

    def gamma_fracintnoise(self,n):
        """ compute the gamma function of a fractionally integrated noise,
        i.e. a process with spectral density 
        f(t) = |1-e^-it|^(-2d)/2pi
        This is used as a correcting term in the compution of truegamma;
        see (13.2.9) page 522 of Brockwell & Davis
        """
        r = zeros(n)
        d = self.d()
        r[0] = scipy.special.gamma(1.-2.*d)/(scipy.special.gamma(1.-d)**2) #gamma(0)
        r[1:] = arange(d,n-1)/arange(1.-d,n-1) # = gamma(h)/gamma(h-1)
        return cumproduct(r)

    def truegamma(self,n):
        """ gamma function corresponding to the true (not inverse) spectral 
        density. Computation is done in 2 steps. 
        """
        g = self.gamma(n,inv=False)
        g +=  exp(sum(self.theta))*self.gamma_fracintnoise(n)
        return g 

    def sim(self,n):
	    """ Simulate a Gaussian vector from the approximation
	    Tn(f^-1) ~ 4pi2 Tn(f)^-1
	    """
	    PrecisionMatrix = toeplitz(self.gamma(n))
	    C = cholesky(PrecisionMatrix,lower=False,overwrite_a=True)
            # Upper triangle, because we compute inv(PrecisionMatrix)
	    return solve(C,randn(n))

    def truesim(self,n):
        """ Simulate a Gaussian Long  Mem process
        n : sample size
        """
        CovMatrix = toeplitz(self.truegamma(n))
        T = cholesky(CovMatrix,lower=True,overwrite_a=True)
        return dot(T,randn(n))
    
    def TrueLogDet(self,g):
        """ computes the exact log Determinant of T_n(f^-1) 
        cost is O(n^3) """
        return 2.*sum(log(diag(cholesky(toeplitz(g)))))
    
    
    def approxLogDet(self,n):
        " Log Det, Approx. method by Brunero "
        
        d = self.d()
        det = ( d**2.*(log(n)+1.5772)+ #n*LOG2PI+  #another blunder by L
            2.*sum(eta*d**arange(3.,21.)) + 2*appro(d) )
        if self.k>0:
            det +=  0.25*sum(arange(1,self.k+1)*self.theta**2)+d*sum(self.theta) 
        return det
        
    
    def WhittlelogLikelihood(self,c):
        """ Computes Whittle's Approx. of log-Likelihood
        c: context 
	periodo: periodogram of the data (needs to be computed only once, 
		first term must be removed)
        """

        n = c.ndata
        quadform = (4.*pi**2)*sum( c.periodogram*self.invspectrum(
            2*pi*arange(1,n+1)/n) )
        # multiply by 4pi^2 because invspectrum returns f^(-1)/4pi^2
        if quadform<=0:
            l = -Inf
        else:
            l  = -0.5*self.approxLogDet(n)-(c.asigma+0.5*n)*log(
                  c.bsigma+0.5*quadform)
        return l

    def TruelogLikelihood(self,c,mean_corrected=True):
        """ true log-likelihood: expansive to compute because it
        requires inversing T(f)
        """

        n = c.ndata
        g = self.truegamma(n)
        T = toeplitz(g)
        if mean_corrected: 
            T += (1./c.gmu)*ones((n,n))
        # this extra term is for marginalising with respect to mu
        C = cho_factor(T,overwrite_a=True)
        ld = sum(log(diag(C[0]))) # log-det of C (hence half logdet of T)
        # C[0] because C is a list of 2 objects, (matrix,flag)
        l = -ld-(c.asigma+0.5*n)*log(
            c.bsigma+0.5*dot(c.y,cho_solve(C,c.y)) )
        return l

    def logPrior(self,c): 
        " Log prior density "
        l = ( c.logpik[self.k] 
              -self.td-2.*log(1.+exp(-self.td)) ) 
            # Logistic prior for td ( <=> U[0,1/2] prior on d ) 
        if self.k>0:
            l += sum( -0.5*(self.theta/c.sigmatheta[:self.k])**2
                    -log(c.sigmatheta[:self.k]) )-self.k*LOGSQRT2PI 
                    # N(0,sig_k) prior on theta_k 
        return l 

    def sample_prior(self,c):
        " generate one point from the prior "
        k, u, s = 0, rand(1), c.pik[0]
        while (k<c.kmax) and (u>s):
            k += 1
            s += c.pik[k]
        self.k = k 
        if k>0:
            self.theta = c.sigmatheta[:k]*randn(k)
        else:
            self.theta = zeros(0)
        self.td = transformd(0.5*rand(1))
        return None


    def RWHM_step(self,c,cov,n=None):
        " One step of Gaussian Random Walk Hasting-Metropolis "
        if n==None:
            n = c.ndata
        allparams = hstack( (self.td,self.theta) )
        proposed_vector = numpy.random.multivariate_normal(allparams,cov)
        proposed_param = Parameter(self.k)
        proposed_param.theta = proposed_vector[1:]
        proposed_param.td = proposed_vector[0]
        proposed_param.llik = proposed_param.WhittlelogLikelihood(c)
        proposed_param.l = proposed_param.llik+ proposed_param.logPrior(c)
        logprob_accept = proposed_param.l-self.l
        if log(rand(1))<logprob_accept:
            return (proposed_param,1)
        else:
            return (self,0)

    def RJBD_step(self,c,n=None):
        " one step of birth-death reversible jump "
        if n==None:
            n = c.ndata
        k = self.k
        birth = True if k==0 else False if k==c.kmax else (rand(1)<0.5)
        if birth: #birth step
            proposed_param = Parameter(k+1)
            proposed_param.theta[:-1] = self.theta[:]
            # add a component from prior
            proposed_param.theta[-1] = c.sigmatheta[k]*randn(1)
            # difference of log prior densities (taking into account change of
            # dim)
            delta_logprior = c.logpik[k+1]-c.logpik[k]
        else: #death step
            proposed_param = Parameter(k-1)
            proposed_param.theta[:] = self.theta[:-1]
            delta_logprior = c.logpik[k-1]-c.logpik[k]
        proposed_param.td = self.td
        proposed_param.llik = proposed_param.WhittlelogLikelihood(c)
        proposed_param.l = proposed_param.llik+proposed_param.logPrior(c)
        logprob_accept = proposed_param.llik -self.llik+delta_logprior
        # some correction neeeded at boundaries k=0 and k=kmax
        if (k==0) or (k==c.kmax): 
            logprob_accept -= LOG2
        if ((k==1) and not(birth)) or ((k==c.kmax-1) and birth):
            logprob_accept += LOG2
        if log(rand(1))<logprob_accept:
            # accept
            return (proposed_param,1)
        else:
            # reject
            return (self,0)
    

def MCMC(c,niter,scaleRW,thinning=100,starting_point=None,n_adapt=0,burnin_adapt=1000,stop_adapt=50000):
    """ run and return a MCMC chain 
    scaleRW: a vector of size (kmax+1), giving the scale of the 
    random walk in each direction (d,theta0,...,theta(kmax))
    adapt: if !=0, adapt progressively the scale of the RW
    (not compatible with RJ moves, and requires to take a small thinning)
    """
    starting_time = time.time()
    N = niter/thinning
    sto_d = zeros(N,double)
    sto_td = zeros_like(sto_d)
    sto_k = zeros(N,int)
    sto_theta = zeros((N,c.kmax),double)
    sto_l = zeros(N)
    covProposal = diag(scaleRW**2)
    if starting_point==None:
        point = Parameter(0)
        point.sample_prior(c) # starting point sampled from prior
    else:
        point = starting_point
        k0 = starting_point.k
    point.l = point.logPrior(c)+point.WhittlelogLikelihood(c)
    naccept, nacceptRJBD, nacceptRJG, j = 0,0,0,0
    for n in xrange(niter): 
        newpoint, acc = point.RWHM_step(c,covProposal[:(point.k+1),:(point.k+1)])
        naccept += acc
        if c.IS_RJBD:
            newerpoint, accRJBD = newpoint.RJBD_step(c)
            nacceptRJBD += accRJBD
            point = newerpoint
        else:
            point = newpoint
        # store values
        if mod(n,thinning) == 0:
            sto_d[j] = point.d()
            sto_td[j] = point.td 
            sto_k[j] = point.k
            sto_theta[j,:point.k] = point.theta[:]
            sto_l[j] = point.l 
            print n, naccept/(1+n), nacceptRJBD/(1+n), sto_l[j], sto_d[j], sto_k[j], sto_theta[j,:point.k]
            j += 1
        # adaption
        if (n>=burnin_adapt) and (n<stop_adapt) and (n_adapt!=0) and mod(n,n_adapt) == 0:
            covProposal[:(k0+1),:(k0+1)] = (2.38**2/k0)*cov( 
                    hstack((sto_td[:j,newaxis],sto_theta[:j,:k0])),rowvar=0 )
    return {'d':sto_d,'k':sto_k,'theta':sto_theta,'l':sto_l,
    'accrates':(naccept/niter,nacceptRJBD/niter),'elapsed_time':time.time()-starting_time }
        
def logmean(x):
    " log(mean(exp(x))) "
    mx = max(x)
    return mx+log(mean(exp(x-mx)))



#****************************************
class Particle(Parameter):
    " a single Particle "
    def __init__(self,k):
        Parameter.__init__(self,k)
        self.pi = 0. # log-prior
        self.lik = 0. #log-likelihood (for n points)
        self.liktarget = 0.
        
    def update_lik(self,c,tempering):
        "compute prior and likelihood for a new particle"
        self.pi = self.logPrior(c)
        self.lik = self.WhittlelogLikelihood(c)
        self.liktarget = tempering*self.lik
        

class ParticleSample:
    " Particle system "

    def __init__(self,c,H):
        self.c = copy.copy(c)
        self.H = H
        self.t = 1  #iteration of the particle filter
        self.tempering = 0. # tempering coefficient
        self.logw = zeros(self.H,double) # Log-weights
        self.w = ones_like(self.logw) # weights, must be computed from self.logw
        # a container for continuous parameters (d, theta)
        self.td = zeros(self.H,double)
        self.theta = zeros((self.H,c.kmax),double)
        self.k = zeros(H,int)
        self.pi = zeros(H,double) # log-prior
        self.lik = zeros(H,double) # log-likelihood (for n points)
        self.liktarget = zeros(H,double) # log-likelihood times temperiing coeff
    
    def GetParamObj(self,j):
        " return particle j as a Particle Object "
        t =  Particle(self.k[j])
        t.theta = self.theta[j,:self.k[j]]
        t.td = self.td[j]
        t.pi = self.pi[j]
        t.lik = self.lik[j]
        t.liktarget = self.liktarget[j] # maybe not necessary ?
        return t    
    
    def SetParamObj(self,j,t):
        " set Parameter obect t as particle ij"
        self.k[j] = t.k
        self.theta[j,:t.k] = t.theta
        self.theta[j,t.k:] = 0.
        self.td[j] = t.td
        self.pi[j] = t.pi
        self.lik[j] = t.lik
        self.liktarget[j] = t.liktarget
    
    def SamplePrior(self):
        " Sample particles from prior"
        s = 0
        c = self.c
        for k in xrange(c.kmax+1):
            if k==c.kmax:
                sizek = self.H-s # fill all the rest with k=kmax
            elif c.IS_POSDIS:
                sizek = self.H//(c.kmax+1)
            else:
                sizek = round(self.H*c.pik[k])
            self.k[s:s+sizek] = k
            wc = c.pik[k]/sizek # so that sum(w)=1
            self.logw[s:s+sizek] = log(wc)
            self.w[s:s+sizek] = wc
            if k>0:
                self.theta[s:s+sizek,0:k] = c.sigmatheta[0:k]*randn(sizek,k)
            s = s+sizek
        self.td[:] = transformd(0.5*rand(self.H))
        for j in xrange(self.H):
            self.pi[j] = self.GetParamObj(j).logPrior(c)
            self.lik[j]= self.GetParamObj(j).WhittlelogLikelihood(c)
        
    def ReweightingStep(self):
        " Reweighting Step "

        def f(alpha,lw,H,c):
            return ESSlog(alpha*lw)-c.threshold_ESS*H
        c,H = self.c,self.H
        if f(1.-self.tempering,self.lik,H,c)>0:
            delta_tempering = 1.-self.tempering
        else:
            delta_tempering = optimize.brentq(
                f,0.,1.-self.tempering,args=(self.lik,H,c))

        log_incremental_weights = delta_tempering*self.lik
        self.logw += log_incremental_weights
        self.w = exp(self.logw-max(self.logw))
        if any(isnan(self.w)):
           raise NameError('weight = NaN')
        self.liktarget += log_incremental_weights
        self.tempering += delta_tempering

        
    def MoveStep(self):
        " Hastings-Metropolis Move Step "
        c = self.c
        # Compute 2 first moments (cond on k)
        fitted_mean,fitted_cov, RW_cov = [], [], [] #empty lists
        allparams = hstack( (self.td[:,newaxis],self.theta) )
        for k in xrange(c.kmax+1):
            Index = (self.k==k)
            if sum(Index)>1: # you need at least 2 points to compute the covariance matrix
                mu = mean(allparams[Index,0:(k+1)],axis=0)
                Sig = ( cov(allparams[Index,0:(k+1)],rowvar=0)+
                          c.eps*diag(hstack((1.,c.sigmatheta[:k]**2))) )
                # add a small fraction of prior variance, to ensure numerical stability 
            else: # take prior moments
                mu = zeros(k+1)
                Sig = diag(hstack((1.,c.sigmatheta[:k]**2)))
            fitted_mean.append(mu.copy())
            fitted_cov.append(Sig.copy())
            RW_cov.append( (c.HMscale/(k+1.))*Sig )

        # Random walk (conditional on k)
        naccept = 0
        for j in xrange(self.H):
            particle = self.GetParamObj(j)
            k = particle.k
            proposed_vector = numpy.random.multivariate_normal(array(allparams[j,:(k+1)]),RW_cov[k])
            proposed_particle = Particle(k)
            proposed_particle.theta[:k] = proposed_vector[1:]
            proposed_particle.td = proposed_vector[0]
            proposed_particle.update_lik(c,self.tempering)
            logprob_accept = ( proposed_particle.pi+proposed_particle.liktarget
                               -particle.pi-particle.liktarget ) 
            if log(rand(1))<logprob_accept: 
                self.SetParamObj(j,proposed_particle)
                naccept += 1
        print "RWHM acceptance rate:", naccept/self.H
        
        # reversible jump (birth and death) step
        naccept_RJBD, nskipped_RJBD = 0, 0 
        if c.IS_RJBD: # reversible jump (birth and death)
            pbirth = zeros(c.kmax+1); pdeath = zeros_like(pbirth)
            if c.adaptRJ:
                pk = self.probk()
                pbirth[0] = pk[1]/(pk[0]+pk[1])
                pdeath[-1] = pk[-2]/(pk[-2]+pk[-1])
                for k in xrange(1,c.kmax):
                    s = sum(pk[k-1:k+2])
                    pbirth[k] = pk[k+1]/s
                    pdeath[k] = pk[k-1]/s
                pbirth[:-1] = clip(pbirth[:-1],0.1,1)
                pdeath[1:] = clip(pdeath[1:],0.1,1)
            else:
                pbirth[0] = 1. 
                pbirth[1:-1] = 0.5
                pdeath[-1] = 1. 
                pdeath[1:-1] = 0.5
            lpbirth = log(pbirth); lpdeath = log(pdeath)
            for j in xrange(self.H):
                particle = self.GetParamObj(j)
                k = particle.k
                u = rand(1)
                birth = (u<pbirth[k])
                death = not(birth) and (u<pbirth[k]+pdeath[k])
                if not(birth) and not(death):
                    nskipped_RJBD += 1 
                    break 
                if birth: #birth step
                    proposed_particle = Particle(k+1)
                    proposed_particle.theta[:-1] = particle.theta[:]
                    proposed_particle.theta[-1] = c.sigmatheta[k]*randn(1) # add a component from prior
                    proposed_particle.td = particle.td
                    part_probacc = c.logpik[k+1]-c.logpik[k]+lpdeath[k+1]-lpbirth[k]
                else: #death step
                    proposed_particle = Particle(k-1)
                    proposed_particle.theta = particle.theta[:-1]
                    proposed_particle.td = particle.td
                    part_probacc = c.logpik[k-1]-c.logpik[k]+lpbirth[k-1]-lpdeath[k]
                proposed_particle.update_lik(c,self.tempering)
                logprob_accept = ( proposed_particle.liktarget
                                    -particle.liktarget+part_probacc)
                # if (k==0) or (k==c.kmax): 
                #     logprob_accept -= LOG2
                # if ((k==1) and not(birth)) or ((k==c.kmax-1) and birth):
                #     logprob_accept += LOG2
                # correction for ratio of probabilities of moves at boundaries (Reversible jump)
                if log(rand(1))<logprob_accept: # if move is accepted
                    self.SetParamObj(j,proposed_particle)
                    naccept_RJBD += 1
            print "RJBD acceptance rate (skipped rate):",  naccept_RJBD/self.H, nskipped_RJBD/self.H
        
        naccept_RJG = 0
        if c.IS_RJG: # reversible jump (Green's strategy)
            fitted_std, fitted_invstd = [], []
            ratiodet = empty(c.kmax)
            for k in xrange(c.kmax+1):
                fitted_std.append( cholesky(fitted_cov[k],lower=True) )
                fitted_invstd.append (inv(fitted_std[k]))
                if k>0:
                    ratiodet[k-1] = ( log(det(fitted_std[k]))
                        -log(det(fitted_std[k-1])) )
            v = empty(k)
            for j in xrange(self.H):
                particle = self.GetParamObj(j)
                k = particle.k
                v = dot(fitted_invstd[k],(hstack((particle.td,particle.theta))- fitted_mean[k]))
                birth = True if k==0 else False if k==c.kmax else (rand(1)>0.5)
                if birth: #birth step
                    u = randn(1)
                    gu = -0.5*u*u-LOGSQRT2PI # N(0,1) density of u
                    propv = ( fitted_mean[k+1] + ravel(dot(fitted_std[k+1],hstack((ravel(v),u)))) )
                    proposed_particle = Particle(k+1)
                    proposed_particle.td = propv[0]
                    proposed_particle.theta = propv[1:]
                    logprob_accept = -gu+ratiodet[k]
                    if k==0:
                        logprob_accept -= LOG2
                    if k==c.kmax-1:
                        logprob_accept += LOG2
                else: #death step
                    u = v[-1]
                    gu = -0.5*u*u-LOGSQRT2PI # N(0,1) density of u
                    propv = ( fitted_mean[k-1]+ravel(dot(fitted_std[k-1],v[:-1])) )
                    proposed_particle = Particle(k-1)
                    proposed_particle.td = propv[0]
                    proposed_particle.theta = propv[1:]
                    logprob_accept = gu-ratiodet[k-1]
                    if k==c.kmax:
                        logprob_accept -= LOG2
                    if k==1:
                        logprob_accept += LOG2
                proposed_particle.update_liks(c,self.n,self.next_n,self.tempering)
                logprob_accept += ( proposed_particle.pi+proposed_particle.liktarget
                                -particle.pi-particle.liktarget )
                if log(rand(1))<logprob_accept: # if move is accepted
                    self.SetParamObj(j,proposed_particle)
                    lp = proposed_particle.logPrior(c)
                    naccept_RJG += 1
   
            print "RJG acceptance rate:", naccept_RJG/self.H
        return (naccept/self.H,naccept_RJBD/self.H,naccept_RJG/self.H)

    def FinalCorrectingStep(self,c):
        """ importance sampling step, from final target
        to true posterior
        Warning: this may be expensive, if n>1000
        """
        
        self.truelik = empty(self.H)
        for j in xrange(self.H):
            particle = self.GetParamObj(j)
            self.truelik[j] = particle.TruelogLikelihood(c)
        dl = self.truelik-self.liktarget
        self.cw = self.w*exp(dl-max(dl))

    def WhittleStep(self,c,n): 
	""" computes the Whittle's approx of the likelihood. 
 	This is just to check that W's approx is crap, and
	that our approx is much better (Lo que queria...)
	"""

	self.whitlik = empty(self.H)
	for j in xrange(self.H):
            particle = self.GetParamObj(j)
            self.whitlik[j] = particle.WhittlelogLikelihood(c,n)
   
    def genESS(self):
        """ Effective Sample Size (generalised version, 
        which takes into account positive discrimination) 
        """
        # not really useful any more in the tempering context
        # because one may compute directly the ESS of inc_weight = {likelihood}^gamma
        
        if self.c.IS_POSDIS: #return average of ESS(k)'s 
            sESS = 0.
            for k in xrange(self.c.kmax):
                sESS += ESS(self.w[self.k==k])
            return sESS
        else:
            return ESS(self.w)
    
    def ResamplingStep(self):
        " Resampling Step "
        if self.c.IS_POSDIS:
            ResampledIndex = zeros(self.H,int)
            j, Hdivk = 0, self.H//(self.c.kmax+1) 
            for k in xrange(self.c.kmax+1):
                Indk = find(self.k==k)
                wk = self.w[Indk]
                ResampledIndex[j:j+Hdivk] = Indk[
                my.IndResample_slow(self.w[Indk],Hdivk) ] 
                pk = sum(wk)/Hdivk # up to a constant, post proba of k 
                self.logw[j:j+Hdivk] = log(pk)
                self.w[j:j+Hdivk] = pk
                j += Hdivk
        else:
            ResampledIndex =  my.IndResample_slow(self.w,self.H)
            self.logw = zeros(self.H) # reset weights 
            self.w = ones_like(self.logw)

        for attr in self.__dict__: # resample each array in the particle system
            if isinstance(self.__dict__[attr], numpy.ndarray) and attr!="logw" and attr!="w":  
                # if is array (and != from weights)
                if self.__dict__[attr].shape[0] == self.H:
                    # if array is of size H in first dimension 
                    self.__dict__[attr] = take(self.__dict__[attr],ResampledIndex,axis=0) 
        
    def probk(self):
        " computes the posterior prob. of k from current particle sample"
        pk = zeros(self.c.kmax+1)
        for i in xrange(pk.size):
            pk[i] = sum(self.w[self.k==i])
        return pk/sum(pk)
    
    
def smc(c,H):
    " main routine "
    starting_time = time.time()
    s = ParticleSample(c,H)
    s.SamplePrior()
    s.sto_ESS = [H,]
    s.sto_probk = []
    s.sto_ar = []
    s.sto_tempering = []
    s.logEvidence = []
    final_step = False
    while not(final_step):
        s.ReweightingStep()
        s.sto_ESS.append( s.genESS() )
        s.sto_probk.append( s.probk() )
        s.sto_tempering.append( s.tempering )
        print s.tempering, s.sto_ESS[-1], sum(invtransformd(s.td)*s.w)/sum(s.w), numpy.round(1000*s.sto_probk[-1])
        s.logEvidence.append( logmean(s.logw) )
        print "Resampling:"
        s.ResamplingStep()
        final_step = abs(1.-s.tempering)<1e-8
        ns = c.nmovesteps_final if final_step else c.nmovesteps
        tab_ar = zeros((ns,3))
        for k in xrange(ns):
            tab_ar[k,:] = s.MoveStep()
        s.sto_ar.append(tab_ar)
    s.d = invtransformd(s.td) # compute d for once before exiting
    s.elapsed_time = time.time()-starting_time
    s.nlikevals = c.nmovesteps*(len(s.sto_tempering)+1)*s.H
    return s
                






