# -*- coding: utf-8 -*-

from scipy import *
from pylab import find, bar
import scipy.weave as weave
#from weave.blitz_tools import blitz_type_factories


# Assume that arrays are of shape [H,k]
# where H is the number of simulations, and k is the parameter dimension
# makes more sense because of the row major order used in Python

# w is an array that contains importance weights, i.e. shape = (H)

def ESS(w):
    " Effective Sample Size "
    return sum(w)**2/sum(w**2)

def wmean(x,w):
    " weighed mean of columns of x (using weights w) "
    return sum(w[:,newaxis]*x,axis=0)/sum(w)
    
def wcov(x,w):
    " weighted covariance matrix of x "
    mu = wmean(x,w)
    A = matrix(w[:,newaxis]*(x-mu))
    return A.T*(x-mu)/sum(w)

def whist(x,w,nbins=None):
    " Weighted histogram of vector x"
    if nbins == None:
        nbins = clip(x.size/10,10,100)
    a, b = x.min(), x.max()
    width = (b-a)/nbins
    xbins = a+width*arange(nbins+1)
    sw = w.sum()
    height = zeros(nbins,double)
    for i in xrange(nbins):
        height[i] = sum( w[find( (x>=xbins[i]) & (x<xbins[i+1]) ) ] )/(sw*width)
    left = xbins[:-1]
    return bar(left,height,width=width)

def IndResample_slow(w,H):
        """ Indexes of resampled particles
        (deterministic scheme) """
        u = rand(1)
        cnw = (H/sum(w))*cumsum(w) # cumsum of normalised weights
        j = 0
        Ind = empty(H,dtype="int")
        for k in range(H):
                while cnw[j]<u:
                        j = j+1
                Ind[k] = j
                u = u + 1.
        return Ind
    

def IndResample(w,H):
    """ Indexes of resampled particles
    (deterministic scheme) """
    
    #NumPy_type = scalar_spec.NumPy_to_blitz_type_mapping[type]
    code = \
    """ int k, j = 0;
         double csw = 0;
        
        for(k=0;k<H;k++)
        {
            while(csw<u)
            {
                j++;
                csw += w(j);
            }
            Ind(k) = j;
            u = u+1.;
        }

    """ #% NumPy_type
    u = rand(1)
    w = double(w*H/sum(w)) # normalised weights
    Ind = empty(H,dtype='int')
    weave.inline(code,['u','H','w','Ind'], 
            type_converters=weave.converters.blitz)
                          #type_factories = blitz_type_factories)
    return Ind
